
#include "BlueLocator.h"
#include <maya/MFnDagNode.h>
#include <maya/MColor.h>

/// the unique ID for our locator node
const MTypeId BlueLocator::typeId( 0x70004 );

/// the typename of our node
const MString BlueLocator::typeName( "blueLocator" );

//------------------------------------------------------------------------
/// \brief	This function is used to render our custom locator node
/// \param	view	-	the maya viewport to render the locator node in
/// \param	DGpath	-	the DAG path of the object
/// \param	style	-
/// \param	status	-
///
void BlueLocator::OnDraw( M3dView::DisplayStyle style,
						M3dView::DisplayStatus status )
{
	// the MPxLocator::colorRGB() function returns the user specified colour for
	// a given selection state. For example, by default a selected item is green;
	// the user may however change this within the user preferences and say change
	// this to pink. In order for our node to maintain continuity with the rest
	// of the users settings, we should use this colour value somewhere obvious
	// on our nodes.
	MColor outlineCol = colorRGB(status);

	glColor3f(0.0f,0.0f,1.0f);

	DrawFilledCube(0.3f,0.3f,0.3f);

	if(status == M3dView::kLead )
		glColor4f(0.0f,1.0f,0.0f,0.5f);
	else
		// Otherwise i use a slightly varied intensity of the colour
		glColor4f(outlineCol.r,outlineCol.g,outlineCol.b,0.5f);

	DrawCrossXY(1,1);
	DrawCrossXZ(1,1);
	DrawCrossYZ(1,1);
}

//------------------------------------------------------------------------
/// \brief	returns the bounding box of the locator node
/// \return	the nodes bounding box
///
MBoundingBox BlueLocator::boundingBox() const {

	// expand the bounding box to fit all axes of the locator node
	MBoundingBox bbox;

	bbox.expand( MPoint( -0.5f, 0.0f, 0.0f ) );
	bbox.expand( MPoint(  0.5f, 0.0f, 0.0f ) );

	bbox.expand( MPoint(  0.0f,-0.5f,  0.0f ) );
	bbox.expand( MPoint(  0.0f, 0.5f,  0.0f ) );

	bbox.expand( MPoint(  0.0f, 0.0f, -0.5f ) );
	bbox.expand( MPoint(  0.0f, 0.0f,  0.5f ) );

	return bbox;
}


//------------------------------------------------------------------------
/// \brief	this function is called by mata to return a new instance of our locator node
/// \return	the new node
///
void* BlueLocator::creator()
{
	return new BlueLocator();
}

//------------------------------------------------------------------------
/// \brief	this function creates a description of our node
/// \return	The status code
///
MStatus BlueLocator::initialize()
{
	return MS::kSuccess;
}
