
#ifndef __CIRCLE_LOCATOR_H__
#define __CIRCLE_LOCATOR_H__

	#ifdef WIN32
		#pragma once
	#endif

	#include "RenderRoutines.h"

	class CircleLocator
		: public RenderRoutines
	{
	public:

		/// \brief	This function is used to render our custom locator node
		/// \param	view	-	the maya viewport to render the locator node in
		/// \param	DGpath	-	the DAG path of the object
		/// \param	style	-	wireframe, shadeded etc
		/// \param	status	-	is the node selected, templated etc
		///
		virtual void OnDraw(  M3dView::DisplayStyle,
							M3dView::DisplayStatus);

		/// \brief	returns the bounding box of the locator node
		/// \return	the nodes bounding box
		virtual MBoundingBox boundingBox() const;

		/// \brief	this function is called by mata to return a new instance of our locator node
		/// \return	the new node
		static void *creator();

		/// \brief	this function creates a description of our node
		/// \return	The status code
		static MStatus initialize();

	// type information
	public:

		/// the unique type ID of our custom node
		static const MTypeId typeId;

		/// the unique type name of our custom node
		static const MString typeName;

	// handles to the attributes added to the node.
	private:

		/// the width & height of the yellow square of the locator
		static MObject m_Radius;


	};

#endif





