
#include "RedLocator.h"
#include "GreenLocator.h"
#include "BlueLocator.h"

#include "RedBox.h"
#include "GreenBox.h"
#include "BlueBox.h"

#include "CircleTarget.h"
#include "QuadTarget.h"
#include "SquareTarget.h"
#include "SpiralTarget.h"

#include "CircleSpawn.h"
#include "QuadSpawn.h"
#include "SquareSpawn.h"
#include "SpiralSpawn.h"

#include "CircleLocator.h"
#include "CrossLocator.h"
#include "QuadLocator.h"
#include "SquareLocator.h"
#include "SpiralLocator.h"

#include "SoftVolumeBox.h"
#include "SoftVolumeCube.h"
#include "SoftVolumeCone.h"
#include "SoftVolumeCylinder.h"
#include "SoftVolumeHourglass.h"
#include "SoftVolumeSphere.h"

#include "VolumeBox.h"
#include "VolumeCube.h"
#include "VolumeCone.h"
#include "VolumeCylinder.h"
#include "VolumeHourglass.h"
#include "VolumeSphere.h"

#include "VolumeTriggerBox.h"
#include "VolumeTriggerCone.h"
#include "VolumeTriggerCube.h"
#include "VolumeTriggerCylinder.h"
#include "VolumeTriggerHourglass.h"
#include "VolumeTriggerSphere.h"

#include <maya/MFnPlugin.h>

// required to link the libraries
#ifdef WIN32
	#pragma comment(lib,"Foundation.lib")
	#pragma comment(lib,"OpenMaya.lib")
	#pragma comment(lib,"OpenMayaFx.lib")
	#pragma comment(lib,"OpenMayaUi.lib")
	#pragma comment(lib,"Image.lib")
	#pragma comment(lib,"OpenMayaAnim.lib")
	#pragma comment(lib,"OpenGl32.lib")
	#pragma comment(lib,"glu32.lib")
#endif

// under WIN32 we have to 'export' these functions so that they are visible
// inside the dll. If maya can't see the functions, it can't load the plugin!
#ifdef WIN32
	#define EXPORT __declspec(dllexport)
#else
	#define EXPORT
#endif

/// \brief	This function is called once when our plugin is loaded. We can use
///			it to initialise any custom nodes, mel functions etc.
/// \param	obj	-	a handle to the loaded plugin
/// \return	an MStatus error code
///
EXPORT MStatus initializePlugin( MObject obj )
{
	MStatus stat;

	// we need to use the plugin function set to register new node types with maya.
	// we can also provide the author of the plugin, the version number, and we
	// can also specify a required version of maya, though usually "Any" will do...
	//
	MFnPlugin fnPlugin( obj, "robthebloke.org", "1.0", "Any");


	//===========================================================	CircleLocator
	stat = fnPlugin.registerNode( CircleLocator::typeName,
								  CircleLocator::typeId,
								  CircleLocator::creator,
								  CircleLocator::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the circleLocator node" );


	//===========================================================	CrossLocator
	stat = fnPlugin.registerNode( CrossLocator::typeName,
								  CrossLocator::typeId,
								  CrossLocator::creator,
								  CrossLocator::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the crossLocator node" );


	//===========================================================	QuadLocator
	stat = fnPlugin.registerNode( QuadLocator::typeName,
								  QuadLocator::typeId,
								  QuadLocator::creator,
								  QuadLocator::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the quadLocator node" );


	//===========================================================	SquareLocator
	stat = fnPlugin.registerNode( SquareLocator::typeName,
								  SquareLocator::typeId,
								  SquareLocator::creator,
								  SquareLocator::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the squareLocator node" );


	//===========================================================	SpiralLocator
	stat = fnPlugin.registerNode( SpiralLocator::typeName,
								  SpiralLocator::typeId,
								  SpiralLocator::creator,
								  SpiralLocator::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the spiralLocator node" );


	//===========================================================	CircleSpawn
	stat = fnPlugin.registerNode( CircleSpawn::typeName,
								  CircleSpawn::typeId,
								  CircleSpawn::creator,
								  CircleSpawn::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the circleSpawn node" );


	//===========================================================	QuadSpawn
	stat = fnPlugin.registerNode( QuadSpawn::typeName,
								  QuadSpawn::typeId,
								  QuadSpawn::creator,
								  QuadSpawn::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the quadSpawn node" );


	//===========================================================	SquareSpawn
	stat = fnPlugin.registerNode( SquareSpawn::typeName,
								  SquareSpawn::typeId,
								  SquareSpawn::creator,
								  SquareSpawn::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the squareSpawn node" );


	//===========================================================	SpiralSpawn
	stat = fnPlugin.registerNode( SpiralSpawn::typeName,
								  SpiralSpawn::typeId,
								  SpiralSpawn::creator,
								  SpiralSpawn::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the spiralSpawn node" );


	//===========================================================	CircleTarget
	stat = fnPlugin.registerNode( CircleTarget::typeName,
								  CircleTarget::typeId,
								  CircleTarget::creator,
								  CircleTarget::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the circleTarget node" );


	//===========================================================	QuadTarget
	stat = fnPlugin.registerNode( QuadTarget::typeName,
								  QuadTarget::typeId,
								  QuadTarget::creator,
								  QuadTarget::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the quadTarget node" );


	//===========================================================	SquareTarget
	stat = fnPlugin.registerNode( SquareTarget::typeName,
								  SquareTarget::typeId,
								  SquareTarget::creator,
								  SquareTarget::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the squareTarget node" );


	//===========================================================	SpiralTarget
	stat = fnPlugin.registerNode( SpiralTarget::typeName,
								  SpiralTarget::typeId,
								  SpiralTarget::creator,
								  SpiralTarget::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the spiralTarget node" );




	//===========================================================	SoftVolumeBox
	stat = fnPlugin.registerNode( SoftVolumeBox::typeName,
								  SoftVolumeBox::typeId,
								  SoftVolumeBox::creator,
								  SoftVolumeBox::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the softVolumeBox node" );

	//===========================================================	SoftVolumeCone
	stat = fnPlugin.registerNode( SoftVolumeCone::typeName,
								  SoftVolumeCone::typeId,
								  SoftVolumeCone::creator,
								  SoftVolumeCone::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the softVolumeCone node" );

	//===========================================================	SoftVolumeCube
	stat = fnPlugin.registerNode( SoftVolumeCube::typeName,
								  SoftVolumeCube::typeId,
								  SoftVolumeCube::creator,
								  SoftVolumeCube::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the softVolumeCube node" );

	//===========================================================	SoftVolumeCylinder
	stat = fnPlugin.registerNode( SoftVolumeCylinder::typeName,
								  SoftVolumeCylinder::typeId,
								  SoftVolumeCylinder::creator,
								  SoftVolumeCylinder::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the softVolumeCylinder node" );

	//===========================================================	SoftVolumeHourGlass
	stat = fnPlugin.registerNode( SoftVolumeHourGlass::typeName,
								  SoftVolumeHourGlass::typeId,
								  SoftVolumeHourGlass::creator,
								  SoftVolumeHourGlass::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the softVolumeHourGlass node" );

	//===========================================================	SoftVolumeSphere
	stat = fnPlugin.registerNode( SoftVolumeSphere::typeName,
								  SoftVolumeSphere::typeId,
								  SoftVolumeSphere::creator,
								  SoftVolumeSphere::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the softVolumeSphere node" );






	//===========================================================	VolumeBox
	stat = fnPlugin.registerNode( VolumeBox::typeName,
								  VolumeBox::typeId,
								  VolumeBox::creator,
								  VolumeBox::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the volumeBox node" );

	//===========================================================	VolumeCone
	stat = fnPlugin.registerNode( VolumeCone::typeName,
								  VolumeCone::typeId,
								  VolumeCone::creator,
								  VolumeCone::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the volumeCone node" );

	//===========================================================	VolumeCube
	stat = fnPlugin.registerNode( VolumeCube::typeName,
								  VolumeCube::typeId,
								  VolumeCube::creator,
								  VolumeCube::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the volumeCube node" );

	//===========================================================	VolumeCylinder
	stat = fnPlugin.registerNode( VolumeCylinder::typeName,
								  VolumeCylinder::typeId,
								  VolumeCylinder::creator,
								  VolumeCylinder::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the volumeCylinder node" );

	//===========================================================	VolumeHourGlass
	stat = fnPlugin.registerNode( VolumeHourGlass::typeName,
								  VolumeHourGlass::typeId,
								  VolumeHourGlass::creator,
								  VolumeHourGlass::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the volumeHourGlass node" );

	//===========================================================	VolumeSphere
	stat = fnPlugin.registerNode( VolumeSphere::typeName,
								  VolumeSphere::typeId,
								  VolumeSphere::creator,
								  VolumeSphere::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the volumeSphere node" );

	//===========================================================	VolumeTriggerBox
	stat = fnPlugin.registerNode( VolumeTriggerBox::typeName,
								  VolumeTriggerBox::typeId,
								  VolumeTriggerBox::creator,
								  VolumeTriggerBox::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the volumeTriggerBox node" );

	//===========================================================	VolumeTriggerCube
	stat = fnPlugin.registerNode( VolumeTriggerCube::typeName,
								  VolumeTriggerCube::typeId,
								  VolumeTriggerCube::creator,
								  VolumeTriggerCube::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the volumeTriggerCube node" );

	//===========================================================	VolumeTriggerCone
	stat = fnPlugin.registerNode( VolumeTriggerCone::typeName,
								  VolumeTriggerCone::typeId,
								  VolumeTriggerCone::creator,
								  VolumeTriggerCone::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the volumeTriggerCone node" );

	//===========================================================	VolumeTriggerCylinder
	stat = fnPlugin.registerNode( VolumeTriggerCylinder::typeName,
								  VolumeTriggerCylinder::typeId,
								  VolumeTriggerCylinder::creator,
								  VolumeTriggerCylinder::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the volumeTriggerCylinder node" );

	//===========================================================	VolumeTriggerHourGlass
	stat = fnPlugin.registerNode( VolumeTriggerHourGlass::typeName,
								  VolumeTriggerHourGlass::typeId,
								  VolumeTriggerHourGlass::creator,
								  VolumeTriggerHourGlass::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the volumeTriggerHourGlass node" );

	//===========================================================	VolumeTriggerSphere
	stat = fnPlugin.registerNode( VolumeTriggerSphere::typeName,
								  VolumeTriggerSphere::typeId,
								  VolumeTriggerSphere::creator,
								  VolumeTriggerSphere::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the volumeTriggerSphere node" );



	//===========================================================	RedLocator
	stat = fnPlugin.registerNode( RedLocator::typeName,
								  RedLocator::typeId,
								  RedLocator::creator,
								  RedLocator::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the redLocator node" );

	//===========================================================	GreenLocator
	stat = fnPlugin.registerNode( GreenLocator::typeName,
								  GreenLocator::typeId,
								  GreenLocator::creator,
								  GreenLocator::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the greenLocator node" );

	//===========================================================	BlueLocator
	stat = fnPlugin.registerNode( BlueLocator::typeName,
								  BlueLocator::typeId,
								  BlueLocator::creator,
								  BlueLocator::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the blueLocator node" );



	//===========================================================	RedBox
	stat = fnPlugin.registerNode( RedBox::typeName,
								  RedBox::typeId,
								  RedBox::creator,
								  RedBox::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the redBox node" );

	//===========================================================	GreenBox
	stat = fnPlugin.registerNode( GreenBox::typeName,
								  GreenBox::typeId,
								  GreenBox::creator,
								  GreenBox::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the greenBox node" );

	//===========================================================	BlueBox
	stat = fnPlugin.registerNode( BlueBox::typeName,
								  BlueBox::typeId,
								  BlueBox::creator,
								  BlueBox::initialize,
								  MPxNode::kLocatorNode );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not register the blueBox node" );



	return stat;
}

/// \brief	This function is called once when our plugin is unloaded. We need
/// 		to tell maya which mel funcs, nodes etc we are removing.
/// \param	obj	-	a handle to the loaded plugin
/// \return	an MStatus error code
///
EXPORT MStatus uninitializePlugin( MObject obj)
{
	MStatus stat;

	// use the plugin function set to unload the plugin
	MFnPlugin fnPlugin( obj );


	//===========================================================	RedLocator
	// deregister the node
	stat = fnPlugin.deregisterNode( RedLocator::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the redLocator node" );


	//===========================================================	GreenLocator
	// deregister the node
	stat = fnPlugin.deregisterNode( GreenLocator::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the greenLocator node" );



	//===========================================================	BlueLocator
	// deregister the node
	stat = fnPlugin.deregisterNode( BlueLocator::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the blueLocator node" );



	//===========================================================	RedBox
	// deregister the node
	stat = fnPlugin.deregisterNode( RedBox::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the redBox node" );


	//===========================================================	GreenBox
	// deregister the node
	stat = fnPlugin.deregisterNode( GreenBox::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the greenBox node" );



	//===========================================================	BlueBox
	// deregister the node
	stat = fnPlugin.deregisterNode( BlueBox::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the blueBox node" );





	//===========================================================	CircleTarget
	// deregister the node
	stat = fnPlugin.deregisterNode( CircleTarget::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the circleTarget node" );


	//===========================================================	QuadTarget
	// deregister the node
	stat = fnPlugin.deregisterNode( QuadTarget::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the quadTarget node" );


	//===========================================================	SquareTarget
	// deregister the node
	stat = fnPlugin.deregisterNode( SquareTarget::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the squareTarget node" );


	//===========================================================	SpiralTarget
	// deregister the node
	stat = fnPlugin.deregisterNode( SpiralTarget::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the spiralTarget node" );



	//===========================================================	CircleSpawn
	// deregister the node
	stat = fnPlugin.deregisterNode( CircleSpawn::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the circleSpawn node" );



	//===========================================================	QuadSpawn
	// deregister the node
	stat = fnPlugin.deregisterNode( QuadSpawn::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the quadSpawn node" );



	//===========================================================	SquareSpawn
	// deregister the node
	stat = fnPlugin.deregisterNode( SpiralSpawn::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the spiralSpawn node" );




	//===========================================================	SquareSpawn
	// deregister the node
	stat = fnPlugin.deregisterNode( SquareSpawn::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the squareSpawn node" );





	//===========================================================	CircleLocator
	// deregister the node
	stat = fnPlugin.deregisterNode( CircleLocator::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the circleLocator node" );



	//===========================================================	CrossLocator
	// deregister the node
	stat = fnPlugin.deregisterNode( CrossLocator::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the crossLocator node" );



	//===========================================================	QuadLocator
	// deregister the node
	stat = fnPlugin.deregisterNode( QuadLocator::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the quadLocator node" );



	//===========================================================	SquareLocator
	// deregister the node
	stat = fnPlugin.deregisterNode( SquareLocator::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the squareLocator node" );



	//===========================================================	SpiralLocator
	// deregister the node
	stat = fnPlugin.deregisterNode( SpiralLocator::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the spiralLocator node" );






	//===========================================================	SoftVolumeBox
	// deregister the node
	stat = fnPlugin.deregisterNode( SoftVolumeBox::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the softVolumeBox node" );


	//===========================================================	SoftVolumeCone
	// deregister the node
	stat = fnPlugin.deregisterNode( SoftVolumeCone::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the softVolumeCone node" );


	//===========================================================	SoftVolumeCube
	// deregister the node
	stat = fnPlugin.deregisterNode( SoftVolumeCube::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the softVolumeCube node" );


	//===========================================================	SoftVolumeCylinder
	// deregister the node
	stat = fnPlugin.deregisterNode( SoftVolumeCylinder::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the softVolumeCylinder node" );


	//===========================================================	SoftVolumeHourGlass
	// deregister the node
	stat = fnPlugin.deregisterNode( SoftVolumeHourGlass::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the softVolumeHourGlass node" );


	//===========================================================	SoftVolumeSphere
	// deregister the node
	stat = fnPlugin.deregisterNode( SoftVolumeSphere::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the softVolumeSphere node" );





	//===========================================================	VolumeTriggerBox
	// deregister the node
	stat = fnPlugin.deregisterNode( VolumeTriggerBox::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the volumeTriggerBox node" );


	//===========================================================	VolumeTriggerCone
	// deregister the node
	stat = fnPlugin.deregisterNode( VolumeTriggerCone::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the volumeTriggerCone node" );



	//===========================================================	VolumeTriggerCube
	// deregister the node
	stat = fnPlugin.deregisterNode( VolumeTriggerCube::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the volumeTriggerCube node" );

	//===========================================================	VolumeTriggerCylinder
	stat = fnPlugin.deregisterNode( VolumeTriggerCylinder::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the volumeTriggerCylinder node" );


	//===========================================================	VolumeTriggerHourGlass
	// deregister the node
	stat = fnPlugin.deregisterNode( VolumeTriggerHourGlass::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the volumeTriggerHourGlass node" );


	//===========================================================	VolumeTriggerSphere
	stat = fnPlugin.deregisterNode( VolumeTriggerSphere::typeId );

	// check for error
	if( stat != MS::kSuccess )
		stat.perror( "could not deregister the volumeTriggerSphere node" );

	return stat;
}
